//--------------------------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//--------------------------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 7
#property indicator_color1 DarkGray
#property indicator_color2 DarkOrange
#property indicator_color3 DarkOrange
#property indicator_color4 LimeGreen
#property indicator_color5 DarkOrange
#property indicator_color6 LimeGreen
#property indicator_color7 DarkOrange
#property indicator_width3 2
#property indicator_style2 STYLE_DOT
#property indicator_style6 STYLE_DOT
#property indicator_style7 STYLE_DOT

#import "fourier extrapolation.dll"
    void fourierExtrapolation(double& source[], double& past[], double& prediction[], int pastBars, int lastBar, int futureBars, int harmonicsNo, double harmonicsTolerance);
#import

//
//
//
//
//

extern int     RsiPeriod   =   14;    
extern int     RsiPrice    =  PRICE_CLOSE;
extern int     RsiMethod   =    2;
extern string  _           = "Methods";
extern string  _0          = "rsi";
extern string  _1          = "Wilders rsi";
extern string  _2          = "rsx";
extern string  _3          = "Cuttler rsi";
extern string  _4          = "T3 rsi";
extern double  T3Hot       =  0.8;
extern bool    T3Original  = false;
extern double  BandsWidth  =    2;
extern int     BandsPeriod =   50;
extern int     BandsMethod =  MODE_SMA;
extern int     LastBar     =    0;    // Last bar in the past data
extern int     PastBars    =  400;    // Number of past bars
extern int     FutBars     =  200;    // Number of bars to predict 
extern int     HarmNo      =    8;    // Number of frequencies; HarmNo=0 computes PastBars harmonics
extern double  FreqTOL     = 0.0001;  // Tolerance of frequency calculation - FreqTOL > 0.001 may not converge

double indicatorValues[];
double pastValues[];
double pastValuesBu[];
double pastValuesBd[];
double futureValues[];
double futureValuesBu[];
double futureValuesBd[];

double source[];
double result[];
double prediction[];

//--------------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------------
//
//
//
//
//

int init()
{
   if(HarmNo==0 || HarmNo>PastBars) HarmNo=PastBars;
      SetIndexBuffer(0,indicatorValues);
      SetIndexBuffer(1,pastValues);      SetIndexLabel(1,"past");
      SetIndexBuffer(2,futureValues);    SetIndexLabel(2,"prediction");
      SetIndexBuffer(3,pastValuesBu);  
      SetIndexBuffer(4,pastValuesBd);  
      SetIndexBuffer(5,futureValuesBu);
      SetIndexBuffer(6,futureValuesBd);
   IndicatorShortName("Fourier extrapolation - of "+getRsiName(RsiMethod));
   return(0);
}
int deinit(){return(0);}

//--------------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         int np = PastBars; if (np>Bars)    np = Bars;
         int nf = FutBars;  if (nf<LastBar) nf = LastBar;

         if (ArraySize(source)     !=np)   ArrayResize(source,np);
         if (ArraySize(result)     !=np)   ArrayResize(result,np);
         if (ArraySize(prediction) !=nf+1) ArrayResize(prediction,nf+1);
               SetIndexShift(1,  -LastBar);
               SetIndexShift(2,nf-LastBar);
               SetIndexShift(3,  -LastBar);
               SetIndexShift(4,  -LastBar);
               SetIndexShift(5,nf-LastBar);
               SetIndexShift(6,nf-LastBar);

      //
      //
      //
      //
      //

      for(int i=limit; i>=0; i--) indicatorValues[i]=iRsi(iMA(NULL,0,1,0,MODE_SMA,RsiPrice,i),RsiPeriod,T3Hot,T3Original,RsiMethod,i);
         double deviation=iStdDevOnArray(indicatorValues,0,BandsPeriod,0,BandsMethod,LastBar);
         ArrayCopy(source,indicatorValues,0,LastBar,np);
               fourierExtrapolation(source,result,prediction,np,LastBar,nf+1,HarmNo,FreqTOL); 
         ArrayCopy(pastValues  ,result); 
         ArrayCopy(futureValues,prediction);
         for(i=np; i>=0; i--) 
         { 
            pastValuesBu[i]=pastValues[i]+BandsWidth*deviation;
            pastValuesBd[i]=pastValues[i]-BandsWidth*deviation;
         }            
         for(i=nf; i>=0; i--) 
         { 
            futureValuesBu[i]=futureValues[i]+BandsWidth*deviation;
            futureValuesBd[i]=futureValues[i]-BandsWidth*deviation;
         }            

      //
      //
      //
      //
      //
      
      SetIndexDrawBegin(1,Bars-np-LastBar);
      SetIndexDrawBegin(2,Bars-nf-1);
      SetIndexDrawBegin(3,Bars-np-LastBar);
      SetIndexDrawBegin(4,Bars-np-LastBar);
      SetIndexDrawBegin(5,Bars-nf-1);
      SetIndexDrawBegin(6,Bars-nf-1);

   return(0); 
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

string rsiMethodNames[] = {"rsi","Wilders rsi","rsx","Cuttler RSI","T3 rsi"};
string getRsiName(int& method)
{
   int max = ArraySize(rsiMethodNames)-1;
      method=MathMax(MathMin(method,max),0); return(rsiMethodNames[method]);
}

//
//
//
//
//

double workRsi[][13];
#define _price  0
#define _change 1
#define _changa 2

double iRsi(double price, double period, double t3Hot, bool t3Original, int rsiMode, int i, int instanceNo=0)
{
   if (ArrayRange(workRsi,0)!=Bars) ArrayResize(workRsi,Bars);
      int z = instanceNo*13; 
      int r = Bars-i-1;
   
   //
   //
   //
   //
   //
   
   workRsi[r][z+_price] = price;
   switch (rsiMode)
   {
      case 0:
         double alpha = 1.0/period; 
         if (r<period)
            {
               int k; double sum = 0; for (k=0; k<period && (r-k-1)>=0; k++) sum += MathAbs(workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price]);
                  workRsi[r][z+_change] = (workRsi[r][z+_price]-workRsi[0][z+_price])/MathMax(k,1);
                  workRsi[r][z+_changa] =                                         sum/MathMax(k,1);
            }
         else
            {
               double change = workRsi[r][z+_price]-workRsi[r-1][z+_price];
                               workRsi[r][z+_change] = workRsi[r-1][z+_change] + alpha*(        change  - workRsi[r-1][z+_change]);
                               workRsi[r][z+_changa] = workRsi[r-1][z+_changa] + alpha*(MathAbs(change) - workRsi[r-1][z+_changa]);
            }
         if (workRsi[r][z+_changa] != 0)
               return(50.0*(workRsi[r][z+_change]/workRsi[r][z+_changa]+1));
         else  return(50.0);
         
      //
      //
      //
      //
      //
      
      case 1 :
         workRsi[r][z+1] = iSmma(0.5*(MathAbs(workRsi[r][z+_price]-workRsi[r-1][z+_price])+(workRsi[r][z+_price]-workRsi[r-1][z+_price])),0.5*(period-1),Bars-i-1,instanceNo*2+0);
         workRsi[r][z+2] = iSmma(0.5*(MathAbs(workRsi[r][z+_price]-workRsi[r-1][z+_price])-(workRsi[r][z+_price]-workRsi[r-1][z+_price])),0.5*(period-1),Bars-i-1,instanceNo*2+1);
         if((workRsi[r][z+1] + workRsi[r][z+2]) != 0) 
               return(100.0 * workRsi[r][z+1]/(workRsi[r][z+1] + workRsi[r][z+2]));
         else  return(50);

      //
      //
      //
      //
      //

      case 2 :     
         double Kg = (3.0)/(2.0+period), Hg = 1.0-Kg;
         if (r<period) { for (k=1; k<13; k++) workRsi[r][k+z] = 0; return(50); }  

         //
         //
         //
         //
         //
      
         double mom = workRsi[r][_price+z]-workRsi[r-1][_price+z];
         double moa = MathAbs(mom);
         for (k=0; k<3; k++)
         {
            int kk = k*2;
            workRsi[r][z+kk+1] = Kg*mom                + Hg*workRsi[r-1][z+kk+1];
            workRsi[r][z+kk+2] = Kg*workRsi[r][z+kk+1] + Hg*workRsi[r-1][z+kk+2]; mom = 1.5*workRsi[r][z+kk+1] - 0.5 * workRsi[r][z+kk+2];
            workRsi[r][z+kk+7] = Kg*moa                + Hg*workRsi[r-1][z+kk+7];
            workRsi[r][z+kk+8] = Kg*workRsi[r][z+kk+7] + Hg*workRsi[r-1][z+kk+8]; moa = 1.5*workRsi[r][z+kk+7] - 0.5 * workRsi[r][z+kk+8];
         }
         if (moa != 0)
              return(MathMax(MathMin((mom/moa+1.0)*50.0,100.00),0.00)); 
         else return(50);
            
      //
      //
      //
      //
      //
      
      case 3 :
         double sump = 0;
         double sumn = 0;
         for (k=0; k<period; k++)
         {
            double diff = workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price];
               if (diff > 0) sump += diff;
               if (diff < 0) sumn -= diff;
         }
         if (sumn > 0)
               return(100.0-100.0/(1.0+sump/sumn));
         else  return(50);
         
      //
      //
      //
      //
      //
               
      case 4 : 
         double chng   = workRsi[r][_price]-workRsi[r-1][_price];
         double changn = iT3(        chng ,period,t3Hot,t3Original,i,instanceNo*2+0);
         double changa = iT3(MathAbs(chng),period,t3Hot,t3Original,i,instanceNo*2+1);
            if (changn != 0)
                  return(MathMin(MathMax(50.0*(changn/MathMax(changa,0.0000001)+1.0),0),100));
            else  return(50.0);
   } 
   return(50.0);
}

//
//
//
//
//
//

double workSmma[][2];
double iSmma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= Bars) ArrayResize(workSmma,Bars);

   //
   //
   //
   //
   //

   if (r<period)
         workSmma[r][instanceNo] = price;
   else  workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//
//
//
//
//

double workT3[][12];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}